#include <iostream>
#include <string>
#include "cursor.cpp"
#include "terminal.cpp"

namespace commands {

/**
 * @brief Prints keys in command like visual input in current line.
 * 
 * @param keys the string to print
 */
void printInput(terminal::state keys) {
    cursor::setX(0);
    std::cout << "\033[2K:" << keys.keys;
}

/**
 * @brief prints the desired output in command-like mode
 * 
 * @param keys keys pressed
 * @param string the output
 */
void printOutput(terminal::state keys, std::wstring string) {
    cursor::setX(0);
    cursor::up();
    std::cout << "\033[2K\n\033[2K:" << keys.keys << "\n";
    std::wcout << L"\033[2K" << string << "\n";
}

/**
 * @brief main loop for commands input.
 * 
 * @return true if we should quit program.
 */
bool loop() {
    std::cout << "\n\033[2K";

    terminal::state keys;

    while(true) {
        printInput(keys);
        auto input = cursor::getchar();

        if (input.code >= 'a' && input.code <= 'z' || input.code >= 'A' && input.code <= 'Z') {
            terminal::key_insert(keys, input.code);
            continue;
        }
        else if (input.code == 8) { // BACKSPACE
            if (!terminal::key_backspace(keys)) return false;
        }
        else if (input.code == 13) { // ENTER
            if (keys.keys.length() == 0) {
                printOutput(keys, L"No command provided");
                break;
            }
            if (keys.keys[0] == 'q') {
                printOutput(keys, L"Exiting...");
                return true;
            }
            else if (keys.keys[0] == 'a') {
                printOutput(keys, L"calculator\nCopyright Rafał F.\nAll rights reserved.");
                break;
            }
            else if (keys.keys[0] == 'h') {
                printOutput(keys, L"Calculator\n:q - quit\n:a - about");
                break;
            }
            else {
                printOutput(keys, L"Command not found\n");
                break;
            }
        }
    }

    return false;
}

} //namespace commands
